#include "MultiGridPreconWrapper.h"
#include "MultiGridSolver.h"
#include "DOFVector.h"

namespace AMDiS {

  MGPreconWrapperScal::MGPreconWrapperScal(::std::string name, int size_, int row_) 
    : PreconditionerScal(size_, row_),
      solution_(NULL)
  {
    mgSolver_ = NEW MultiGridSolverScal(name);
  }

  MGPreconWrapperScal::~MGPreconWrapperScal()
  {
    if(solution_) {
      DELETE solution_;
    }
    DELETE mgSolver_;
  }

  void MGPreconWrapperScal::init() {
    FUNCNAME("MGPreconWrapperScal::init");
    TEST_EXIT(matrix[row])("no matrix\n");
    initMG_ = true;
  };

  void MGPreconWrapperScal::exit() {
    mgSolver_->exitMultiGrid();
  };

  void MGPreconWrapperScal::precon(DOFVector<double> *vec)
  {
    if(!solution_) {
      solution_ = NEW DOFVector<double>(vec->getFESpace(), "MG precon solution");
    }

    solution_->set(0.0);
    mgSolver_->solve((*(*(matrix[row]))), *solution_, *vec, initMG_, false);
    initMG_ = false;

    vec->copy(*solution_);
  }

}
